{% extends "base.html" %}

{% set title = "Riego - Ventilboxen" %}
{% set active_menu = "boxes" %}


{% block content %}
<h1>New Item</h1>
<form action="{{url('boxes_new')}}" method="POST"
      enctype="multipart/form-data">

      <div class="mb-3">
        <label for="id1" class="form-label">Topic (unique)</label>
        <input id="id1" name="topic" type="text" class="form-control">
      </div>


      <div class="mb-3">
        <label for="id2" class="form-label">Name</label>
        <input id="id2" name="name"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id3" class="form-label">Remark</label>
        <input id="id3" name="remark"  type="text" class="form-control">
      </div>

      <input type="submit">
</form>
<hr>
<p>
    <a href="{{url('boxes')}}">list</a>
</p>
{% endblock %}
