{% extends "base.html" %}

{% set title = "Riego - Bewässerungskreise" %}
{% set active_menu = "valves" %}


{% block content %}
<h1>Edit: {{ item['name'] }}</h1>
<form action="{{url('valves_item_edit', item_id=item['id']) }}" method="POST"
      enctype="multipart/form-data">

      <div class="mb-3">
        <label for="id1" class="form-label">name</label>
        <input id="id1" name="name" value="{{item['name']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id2" class="form-label">remark</label>
        <input id="id2" name="remark" value="{{item['remark']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id3" class="form-label">duration</label>
        <input id="id43" name="duration" value="{{item['duration']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id4" class="form-label">interval</label>
        <input id="id4" name="interval" value="{{item['interval']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id5" class="form-label">is_running</label>
        <input id="id5" name="is_running" value="{{item['is_running']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id5" class="form-label">is_enabled</label>
        <input id="id5" name="is_enabled" value="{{item['is_enabled']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id6" class="form-label">is_hidden</label>
        <input id="id6" name="is_hidden" value="{{item['is_hidden']}}"  type="text" class="form-control">
      </div>

      <div class="mb-3">
        <label for="id7" class="form-label">prio</label>
        <input id="id7" name="prio" value="{{item['prio']}}"  type="text" class="form-control">
      </div>


  <input type="submit">
</form>
<hr>

<p>
  <a href="{{url('valves')}}">list</a>
  <a href="{{url('valves_item_view', item_id=item['id']) }}">view</a>
  <a href="{{url('valves_item_delete', item_id=item['id']) }}">delete</a>
</p>
{% endblock %}
